@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2004 John Ellis
+ * (C) 2006 John Ellis
  *
  * Author: John Ellis
  *
@@ -23,6 +23,8 @@
 #include "image-overlay.h"
 #include "info.h"
 #include "menu.h"
+#include "pixbuf_util.h"
+#include "print.h"
 #include "slideshow.h"
 #include "utilops.h"
 #include "ui_bookmark.h"
@@ -31,8 +33,6 @@
 
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
 
-#include "icons/view.xpm"
-
 
 typedef struct _ViewWindow ViewWindow;
 struct _ViewWindow
@@ -325,150 +325,52 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 {
 	ViewWindow *vw = data;
 	ImageWindow *imd;
-	gint stop_signal = FALSE;
+	gint stop_signal;
 	GtkWidget *menu;
 	gint x = 0;
 	gint y = 0;
 
 	imd = view_window_active_image(vw);
 
+	stop_signal = TRUE;
 	switch (event->keyval)
 		{
 		case GDK_Left: case GDK_KP_Left:
 			x -= 1;
-			stop_signal = TRUE;
 			break;
 		case GDK_Right: case GDK_KP_Right:
 			x += 1;
-			stop_signal = TRUE;
 			break;
 		case GDK_Up: case GDK_KP_Up:
 			y -= 1;
-			stop_signal = TRUE;
 			break;
 		case GDK_Down: case GDK_KP_Down:
 			y += 1;
-			stop_signal = TRUE;
+			break;
+		default:
+			stop_signal = FALSE;
 			break;
 		}
 
-	if ( !(event->state & GDK_CONTROL_MASK) )
-	    switch (event->keyval)
+	if (x != 0 || y!= 0)
 		{
-		case GDK_Page_Up: case GDK_KP_Page_Up:
-		case GDK_BackSpace:
-		case 'B': case 'b':
-			view_step_prev(vw);
-			stop_signal = TRUE;
-			break;
-		case GDK_Page_Down: case GDK_KP_Page_Down:
-		case GDK_space:
-		case 'N': case 'n':
-			view_step_next(vw);
-			stop_signal = TRUE;
-			break;
-		case GDK_Home: case GDK_KP_Home:
-			view_step_to_end(vw, FALSE);
-			stop_signal = TRUE;
-			break;
-		case GDK_End: case GDK_KP_End:
-			view_step_to_end(vw, TRUE);
-			stop_signal = TRUE;
-			break;
-		case '+': case '=': case GDK_KP_Add:
-			image_zoom_adjust(imd, get_zoom_increment());
-			break;
-		case '-': case GDK_KP_Subtract:
-			image_zoom_adjust(imd, -get_zoom_increment());
-			break;
-		case 'X': case 'x': case GDK_KP_Multiply:
-			image_zoom_set(imd, 0.0);
-			break;
-		case 'Z': case 'z': case GDK_KP_Divide: case '1':
-			image_zoom_set(imd, 1.0);
-			break;
-		case '2':
-			image_zoom_set(imd, 2.0);
-			break;
-		case '3':
-			image_zoom_set(imd, 3.0);
-			break;
-		case '4':
-			image_zoom_set(imd, 4.0);
-			break;
-		case '7':
-			image_zoom_set(imd, -4.0);
-			break;
-		case '8':
-			image_zoom_set(imd, -3.0);
-			break;
-		case '9':
-			image_zoom_set(imd, -2.0);
-			break;
-		case 'W': case 'w':
-			image_zoom_set_fill_geometry(imd, FALSE);
-			break;
-		case 'H': case 'h':
-			image_zoom_set_fill_geometry(imd, TRUE);
-			break;
-		case 'R': case 'r':
-			image_reload(imd);
-			break;
-		case 'S': case 's':
-			if (vw->ss)
-				view_slideshow_stop(vw);
-			else
-				view_slideshow_start(vw);
-			stop_signal = TRUE;
-			break;
-		case 'P': case 'p':
-			slideshow_pause_toggle(vw->ss);
-			break;
-		case 'F': case 'f':
-		case 'V': case 'v':
-			view_fullscreen_toggle(vw, FALSE);
-			stop_signal = TRUE;
-			break;
-		case 'I': case 'i':
-			view_overlay_toggle(vw);
-			break;
-		case ']':
-			image_alter(imd, ALTER_ROTATE_90);
-			stop_signal = TRUE;
-			break;
-		case '[':
-			image_alter(imd, ALTER_ROTATE_90_CC);
-			stop_signal = TRUE;
-			break;
-		case GDK_Delete: case GDK_KP_Delete:
-			if (enable_delete_key)
-				{
-				file_util_delete(image_get_path(imd), NULL, imd->widget);
-				stop_signal = TRUE;
-				}
-			break;
-		case GDK_Escape:
-			if (vw->fs)
-				{
-				view_fullscreen_toggle(vw, TRUE);
-				}
-			else
-				{
-				gtk_widget_destroy(vw->window);
-				}
-			stop_signal = TRUE;
-			break;
-		case GDK_Menu:
-		case GDK_F10:
-			menu = view_popup_menu(vw);
-			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, view_window_menu_pos_cb, vw, 0, GDK_CURRENT_TIME);
-			stop_signal = TRUE;
-			break;
+		if (event->state & GDK_SHIFT_MASK)
+			{
+			x *= 3;
+			y *= 3;
+			}
+
+		keyboard_scroll_calc(&x, &y, event);
+		image_scroll(imd, x, y);
 		}
 
+	if (stop_signal) return stop_signal;
+
 	if (event->state & GDK_CONTROL_MASK)
 		{
 		gint n = -1;
+
+		stop_signal = TRUE;
 		switch (event->keyval)
 			{
 			case '1':
@@ -503,27 +405,25 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 				break;
 			case 'C': case 'c':
 				file_util_copy(image_get_path(imd), NULL, NULL, imd->widget);
-				stop_signal = TRUE;
 				break;
 			case 'M': case 'm':
 				file_util_move(image_get_path(imd), NULL, NULL, imd->widget);
-				stop_signal = TRUE;
 				break;
 			case 'R': case 'r':
 				file_util_rename(image_get_path(imd), NULL, imd->widget);
-				stop_signal = TRUE;
 				break;
 			case 'D': case 'd':
 				file_util_delete(image_get_path(imd), NULL, imd->widget);
-				stop_signal = TRUE;
 				break;
 			case 'P': case 'p':
 				info_window_new(image_get_path(imd), NULL);
-				stop_signal = TRUE;
 				break;
 			case 'W': case 'w':
 				view_window_close(vw);
 				break;
+			default:
+				stop_signal = FALSE;
+				break;
 			}
 		if (n != -1)
 			{
@@ -537,31 +437,148 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 		}
 	else if (event->state & GDK_SHIFT_MASK)
 		{
+		stop_signal = TRUE;
 		switch (event->keyval)
 			{
 			case 'R': case 'r':
 				image_alter(imd, ALTER_ROTATE_180);
-				stop_signal = TRUE;
 				break;
 			case 'M': case 'm':
 				image_alter(imd, ALTER_MIRROR);
-				stop_signal = TRUE;
 				break;
 			case 'F': case 'f':
 				image_alter(imd, ALTER_FLIP);
-				stop_signal = TRUE;
+				break;
+			case 'P': case 'p':
+				{
+				const gchar *path;
+
+				view_fullscreen_toggle(vw, TRUE);
+				imd = view_window_active_image(vw);
+				path = image_get_path(imd);
+				print_window_new(path,
+						 (path) ? g_list_append(NULL, g_strdup(path)) : NULL,
+						 path_list_copy(vw->list), vw->window);
+				}
 				break;
 			default:
+				stop_signal = FALSE;
 				break;
 			}
-		x *= 3;
-		y *= 3;
 		}
-
-	if (x != 0 || y!= 0)
+	else 
 		{
-		keyboard_scroll_calc(&x, &y, event);
-		image_scroll(imd, x, y);
+		stop_signal = TRUE;
+		switch (event->keyval)
+			{
+			case GDK_Page_Up: case GDK_KP_Page_Up:
+			case GDK_BackSpace:
+			case 'B': case 'b':
+				view_step_prev(vw);
+				break;
+			case GDK_Page_Down: case GDK_KP_Page_Down:
+			case GDK_space:
+			case 'N': case 'n':
+				view_step_next(vw);
+				break;
+			case GDK_Home: case GDK_KP_Home:
+				view_step_to_end(vw, FALSE);
+				break;
+			case GDK_End: case GDK_KP_End:
+				view_step_to_end(vw, TRUE);
+				break;
+			case '+': case '=': case GDK_KP_Add:
+				image_zoom_adjust(imd, get_zoom_increment());
+				break;
+			case '-': case GDK_KP_Subtract:
+				image_zoom_adjust(imd, -get_zoom_increment());
+				break;
+			case 'X': case 'x': case GDK_KP_Multiply:
+				image_zoom_set(imd, 0.0);
+				break;
+			case 'Z': case 'z': case GDK_KP_Divide: case '1':
+				image_zoom_set(imd, 1.0);
+				break;
+			case '2':
+				image_zoom_set(imd, 2.0);
+				break;
+			case '3':
+				image_zoom_set(imd, 3.0);
+				break;
+			case '4':
+				image_zoom_set(imd, 4.0);
+				break;
+			case '7':
+				image_zoom_set(imd, -4.0);
+				break;
+			case '8':
+				image_zoom_set(imd, -3.0);
+				break;
+			case '9':
+				image_zoom_set(imd, -2.0);
+				break;
+			case 'W': case 'w':
+				image_zoom_set_fill_geometry(imd, FALSE);
+				break;
+			case 'H': case 'h':
+				image_zoom_set_fill_geometry(imd, TRUE);
+				break;
+			case 'R': case 'r':
+				image_reload(imd);
+				break;
+			case 'S': case 's':
+				if (vw->ss)
+					{
+					view_slideshow_stop(vw);
+					}
+				else
+					{
+					view_slideshow_start(vw);
+					}
+				break;
+			case 'P': case 'p':
+				slideshow_pause_toggle(vw->ss);
+				break;
+			case 'F': case 'f':
+			case 'V': case 'v':
+			case GDK_F11:
+				view_fullscreen_toggle(vw, FALSE);
+				break;
+			case 'I': case 'i':
+				view_overlay_toggle(vw);
+				break;
+			case ']':
+				image_alter(imd, ALTER_ROTATE_90);
+				break;
+			case '[':
+				image_alter(imd, ALTER_ROTATE_90_CC);
+				break;
+			case GDK_Delete: case GDK_KP_Delete:
+				if (enable_delete_key)
+					{
+					file_util_delete(image_get_path(imd), NULL, imd->widget);
+					}
+				break;
+			case GDK_Escape:
+				if (vw->fs)
+					{
+					view_fullscreen_toggle(vw, TRUE);
+					}
+				else
+					{
+					view_window_close(vw);
+					}
+				break;
+			case GDK_Menu:
+			case GDK_F10:
+				menu = view_popup_menu(vw);
+				gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+					       view_window_menu_pos_cb, vw, 0, GDK_CURRENT_TIME);
+				break;
+			default:
+				stop_signal = FALSE;
+				break;
+			}
 		}
 
 	return stop_signal;
@@ -758,17 +775,26 @@ static void view_slideshow_stop(ViewWindow *vw)
 	if (vw->ss) slideshow_free(vw->ss);
 }
 
-static void view_window_close(ViewWindow *vw)
+static void view_window_destroy_cb(GtkWidget *widget, gpointer data)
 {
+	ViewWindow *vw = data;
+
 	view_window_list = g_list_remove(view_window_list, vw);
 
 	view_slideshow_stop(vw);
-	view_fullscreen_toggle(vw, TRUE);
-	gtk_widget_destroy(vw->window);
+	fullscreen_stop(vw->fs);
+
 	path_list_free(vw->list);
 	g_free(vw);
 }
 
+static void view_window_close(ViewWindow *vw)
+{
+	view_slideshow_stop(vw);
+	view_fullscreen_toggle(vw, TRUE);
+	gtk_widget_destroy(vw->window);
+}
+
 static gint view_window_delete_cb(GtkWidget *w, GdkEventAny *event, gpointer data)
 {
 	ViewWindow *vw = data;
@@ -805,7 +831,7 @@ static ViewWindow *real_view_window_new(const gchar *path, GList *list, Collecti
         gtk_window_set_wmclass(GTK_WINDOW(vw->window), "view", "GQview");
         gtk_container_set_border_width(GTK_CONTAINER(vw->window), 0);
 
-	window_set_icon(vw->window, (const gchar **)view_xpm, NULL);
+	window_set_icon(vw->window, PIXBUF_INLINE_ICON_VIEW, NULL);
 
 	vw->imd = image_new(FALSE);
 
@@ -823,6 +849,8 @@ static ViewWindow *real_view_window_new(const gchar *path, GList *list, Collecti
 
 	view_image_set_buttons(vw, vw->imd);
 
+	g_signal_connect(G_OBJECT(vw->window), "destroy",
+			 G_CALLBACK(view_window_destroy_cb), vw);
 	g_signal_connect(G_OBJECT(vw->window), "delete_event",
 			 G_CALLBACK(view_window_delete_cb), vw);
 	g_signal_connect(G_OBJECT(vw->window), "key_press_event",
