@@ -28,7 +28,6 @@
 #include "ui_misc.h"
 #include "ui_tabcomp.h"
 
-#include "icons/tools.xpm"
 
 #define MAINWINDOW_DEF_WIDTH 700
 #define MAINWINDOW_DEF_HEIGHT 500
@@ -262,9 +261,15 @@ static void layout_sort_button_press_cb(GtkWidget *widget, gpointer data)
 
 	menu = submenu_add_sort(NULL, G_CALLBACK(layout_sort_menu_cb), lw, FALSE, FALSE, TRUE, lw->sort_method);
 
-	/* apparently the menu is never sunk, even on a popup */
+	/* take ownership of menu */
+#ifdef GTK_OBJECT_FLOATING
+	/* GTK+ < 2.10 */
 	g_object_ref(G_OBJECT(menu));
 	gtk_object_sink(GTK_OBJECT(menu));
+#else
+	/* GTK+ >= 2.10 */
+	g_object_ref_sink(G_OBJECT(menu));
+#endif
 
         /* ascending option */
 	menu_item_add_divider(menu);
@@ -1105,7 +1110,7 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
         	gtk_window_set_wmclass(GTK_WINDOW(lw->tools), "tools", "GQview");
         	gtk_container_set_border_width(GTK_CONTAINER(lw->tools), 0);
 
-		window_set_icon(lw->tools, (const gchar **)tools_xpm, NULL);
+		window_set_icon(lw->tools, PIXBUF_INLINE_ICON_TOOLS, NULL);
 
 		new_window = TRUE;
 		}
@@ -1709,8 +1714,16 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 #endif
 
 	lw->tooltips = gtk_tooltips_new();
+
+	/* take ownership of tooltips */
+#ifdef GTK_OBJECT_FLOATING
+	/* GTK+ < 2.10 */
 	g_object_ref(G_OBJECT(lw->tooltips));
 	gtk_object_sink(GTK_OBJECT(lw->tooltips));
+#else
+	/* GTK+ >= 2.10 */
+	g_object_ref_sink(G_OBJECT(lw->tooltips));
+#endif
 
 	lw->main_box = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(lw->window), lw->main_box);
