@@ -153,7 +153,7 @@ static gchar *thumb_std_cache_path(const gchar *path, const gchar *uri, gint loc
 		cache_base = g_strconcat(homedir(), "/", THUMB_FOLDER, "/", cache_subfolder, NULL);
 		}
 
-	md5_get_digest(uri, strlen(uri), digest);
+	md5_get_digest((guchar *)uri, strlen(uri), digest);
 	md5_text = md5_digest_to_text(digest);
 
 	if (cache_base && md5_text)
@@ -714,14 +714,14 @@ gint thumb_loader_std_start(ThumbLoaderStd *tl, const gchar *path)
 		{
 		gint found;
 
-		if (thumb_loader_std_fail_check(tl)) return FALSE;
-
 		tl->thumb_path = thumb_loader_std_cache_path(tl, FALSE, NULL, FALSE);
 		tl->thumb_path_local = FALSE;
 
 		found = isfile(tl->thumb_path);
 		if (found && thumb_loader_std_setup(tl, tl->thumb_path)) return TRUE;
 
+		if (thumb_loader_std_fail_check(tl)) return FALSE;
+
 		return thumb_loader_std_next_source(tl, found);
 		}
 
@@ -1117,7 +1117,7 @@ void thumb_std_maint_moved(const gchar *source, const gchar *dest)
 
 	if (thumb_std_maint_move_tail)
 		{
-		g_list_append(thumb_std_maint_move_tail, tm);
+		thumb_std_maint_move_tail = g_list_append(thumb_std_maint_move_tail, tm);
 		thumb_std_maint_move_tail = thumb_std_maint_move_tail->next;
 		}
 	else
